/*
 * LMDB Abstraction Layer.
 * Copyright (C) 2023  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LMDBAL_CURSOR_H
#define LMDBAL_CURSOR_H

#include <string>

#include "lmdb.h"
#include "base.h"
#include "storage.h"

namespace LMDBAL {

template <class K, class V>
class Cursor {
    friend class Storage<K, V>;
private:
    enum State {            /**<Cursor state:*/
        closed,             /**<    - closed*/
        openedPublic,       /**<    - opened with public transaction, all storages will be notified about it after it's done*/
        openedPrivate       /**<    - opened with private transaction, only current storage will be notified when cursor is closed*/
    };

    Cursor(Storage<K, V>* parent);
    ~Cursor();

public:
    void open() const;
    void open(TransactionID txn) const;
    void renew() const;
    void renew(TransactionID txn) const;
    void close() const;

    std::pair<K, V> first() const;
    std::pair<K, V> last() const;
    std::pair<K, V> next() const;
    std::pair<K, V> prev() const;
    std::pair<K, V> current() const;

    void first(std::pair<K, V>& out) const;
    void last(std::pair<K, V>& out) const;
    void next(std::pair<K, V>& out) const;
    void prev(std::pair<K, V>& out) const;
    void current(std::pair<K, V>& out) const;

private:
    void terminated() const;

private:
    Storage<K, V>* storage;
    mutable MDB_cursor* cursor;
    mutable State state;

    inline static const std::string openRecordMethodName = "Cursor::open";   /**<\brief member function name, just for exceptions*/
    inline static const std::string closeRecordMethodName = "Cursor::close"; /**<\brief member function name, just for exceptions*/
    inline static const std::string renewRecordMethodName = "Cursor::renew"; /**<\brief member function name, just for exceptions*/
};

};


#include "cursor.hpp"

#endif //LMDBAL_CURSOR_H
